/*
 *  Copyright Droids Corporation
 *  Olivier Matz <zer0@droids-corp.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: cmdline.h,v 1.4 2009-11-08 17:24:33 zer0 Exp $
 *
 */


#ifdef USE_USB
#include "DualVirtualSerial.h"
#else
#define CMDLINE_UART 0
#define XBEE_UART 2
#endif

void cmdline_init(void);

/* uart rx callback for reset() */
void emergency(char c);

/* log function */
void mylog(struct error * e, ...);

/* poll cmdline */
int cmdline_poll(void);

int cmdline_dev_send(char c, FILE* f);
int cmdline_dev_recv(FILE* f);

int xbee_dev_send(char c, FILE* f);
int xbee_dev_recv(FILE* f);

static inline uint8_t cmdline_keypressed(void) {
#ifdef USE_USB
	return (CDC_Device_ReceiveByte(&VirtualSerial1_CDC_Interface) >= 0);
#else
	return (uart_recv_nowait(CMDLINE_UART) != -1);
#endif
}

static inline int16_t cmdline_getchar(void) {
#ifdef USE_USB
	return CDC_Device_ReceiveByte(&VirtualSerial1_CDC_Interface);
#else
	return uart_recv_nowait(CMDLINE_UART);
#endif
}

static inline uint8_t cmdline_getchar_wait(void) {
#ifdef USE_USB
	int16_t c;
	do {
		c = CDC_Device_ReceiveByte(&VirtualSerial1_CDC_Interface);
	} while (c < 0);
	return c;
#else
	return uart_recv(CMDLINE_UART);
#endif
}
